#!/usr/bin/python
# -*- coding: utf-8 -*-

# use eAVControl instead of eAVSwitch
# all remake for py3 and major fix from @lululla 20240915
from .picplayer import sgrabberPic_Thumb
from . import _
try:
    from Components.AVSwitch import AVSwitch
except ImportError:
    from Components.AVSwitch import eAVControl as AVSwitch
from Components.ActionMap import ActionMap
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.MenuList import MenuList
from Components.Pixmap import Pixmap
from Components.Sources.StaticText import StaticText
from Components.config import (
    config,
    getConfigListEntry,
    ConfigSubsection,
    ConfigSelection,
    configfile,
)
from Plugins.Plugin import PluginDescriptor
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from GlobalActions import globalActionMap
from Screens.Standby import TryQuitMainloop
from Tools.Directories import fileExists, copyfile
from Tools.Notifications import AddNotification
from datetime import datetime
from time import time as systime
from keymapparser import readKeymap
import enigma
import sys
import os
from enigma import (
    getDesktop,
    ePicLoad,
)
from os import (
    remove,
    listdir,
    path,
    makedirs,
)

PY3 = sys.version_info[0] == 3

global globalActionMap
size_w = getDesktop(0).size().width()
size_h = getDesktop(0).size().height()


def makedir(path):
    try:
        makedirs(path)
        return True
    except:
        return False


def getScale():
    return AVSwitch().getFramebufferScale()


def getMountedDevs():

    from Tools.Directories import resolveFilename, SCOPE_MEDIA
    from Components.Harddisk import harddiskmanager

    def handleMountpoint(loc):
        mp = loc[0]
        desc = loc[1]
        return (mp, desc + ' (' + mp + ')')

    mountedDevs = [(resolveFilename(SCOPE_MEDIA, 'hdd'), _('Harddisk')),
                   (resolveFilename(SCOPE_MEDIA, 'usb'), _('USB Device'))]
    mountedDevs += [(p.mountpoint, _(p.description) if p.description else '') for p in harddiskmanager.getMountedPartitions(True)]
    mountedDevs = [path for path in mountedDevs if os.path.isdir(path[0]) and os.access(path[0], os.W_OK | os.X_OK)]
    netDir = resolveFilename(SCOPE_MEDIA, 'net')
    if os.path.isdir(netDir):
        mountedDevs += [(os.path.join(netDir, p), _('Network mount')) for p in os.listdir(netDir)]
    mountedDevs += [(os.path.join('/', 'tmp'), _('Tmp Folder'))]
    mountedDevs = list(map(handleMountpoint, mountedDevs))
    return mountedDevs


currversion = '2.8'
config.plugins.sgrabber = ConfigSubsection()
config.plugins.sgrabber.fixedaspectratio = ConfigSelection(default='Disabled', choices=[('-n', _('Enabled')), ('Disabled', _('Off'))])
config.plugins.sgrabber.always43 = ConfigSelection(default='Disabled', choices=[('-l', _('Enabled')), ('Disabled', _('Off'))])
config.plugins.sgrabber.bicubic = ConfigSelection(default='Disabled', choices=[('-b', _('Enabled')), ('Disabled', _('Off'))])
config.plugins.sgrabber.storedir = ConfigSelection(choices=getMountedDevs())
config.plugins.sgrabber.formatp = ConfigSelection(default='-p', choices=[('-j 100', _('jpg100')),
                                                                         ('-j 80', _('jpg80')),
                                                                         ('-j 60', _('jpg60')),
                                                                         ('-j 40', _('jpg40')),
                                                                         ('-j 20', _('jpg20')),
                                                                         ('-j 10', _('jpg20')),
                                                                         ('bmp', _('BMP')),
                                                                         ('-p', _('PNG'))])
config.plugins.sgrabber.items = ConfigSelection(default='Disabled', choices=[('All', _('Grab All')),
                                                                             ('-v', _('Video only')),
                                                                             ('-o', _('OSD only')),
                                                                             ('Disabled', _('Off'))])
config.plugins.sgrabber.newsize = ConfigSelection(default='Disabled', choices=[('-r1920', _('1920*1080')),
                                                                               ('-r800', _('800*450')),
                                                                               ('-r600', _('600*337')),
                                                                               ('Disabled', _('Skin resolution'))])
config.plugins.sgrabber.scut = ConfigSelection(default='green', choices=[('text', _('Text')),
                                                                         ('help', _('Help')),
                                                                         ('info', _('Info')),
                                                                         ('green', _('Long green')),
                                                                         ('red', _('Long red')),
                                                                         ('video', _('Video')),
                                                                         ('mute', _('Mute')),
                                                                         ('radio', _('Radio'))])

srootfolder = config.plugins.sgrabber.storedir.value
if path.exists(srootfolder + "/screenshots/"):
    print('srootfolder + /screenshots/ exist')
else:
    makedirs(srootfolder + "/screenshots/")
    print('makedir: srootfolder + /screenshots/ exist')


class GrabPreview(Screen):

    skin = '<screen flags="wfNoBorder" position="0,0" size="%d,%d" title="GrabPreview" backgroundColor="#ff0000">' % (size_w, size_h)
    skin += ' <widget name="pixmap" position="0,0" size="%d,%d" zPosition="1" alphatest="on" />' % (size_w, size_h)
    skin += ' <eLabel font="Regular;24" backgroundColor="#00640808" halign="center" valign="center" transparent="0" position="17,15" size="300,55" text="Please wait....." zPosition="9" />\n'
    skin += ' <eLabel name="" position="797,15" size="160,55" halign="center" valign="center" transparent="0" cornerRadius="26" font="Regular; 22" zPosition="9" text="Exit=Play TV" backgroundColor="#01080911" foregroundColor="#fcc000" />\n'
    skin += '</screen>'

    def __init__(self, session, previewPng=None):
        self.skin = GrabPreview.skin
        Screen.__init__(self, session)
        self.session = session
        self.Scale = getScale()
        self.PicLoad = ePicLoad()
        self.previewPng = previewPng
        self['pixmap'] = Pixmap()
        try:
            self.PicLoad.PictureData.get().append(self.DecodePicture)
        except:
            self.PicLoad_conn = self.PicLoad.PictureData.connect(self.DecodePicture)
        self['actions'] = ActionMap(['OkCancelActions',
                                     'ColorActions'], {'ok': self.close,
                                                       'cancel': self.close,
                                                       'blue': self.close}, -1)
        self.onLayoutFinish.append(self.ShowPicture)

    def ShowPicture(self):
        # print('ShowPicture')
        if self.previewPng is not None:
            self.PicLoad.setPara([self['pixmap'].instance.size().width(),
                                  self['pixmap'].instance.size().height(),
                                  self.Scale[0],
                                  self.Scale[1],
                                  0,
                                  1,
                                  '#00080808'])
            self.PicLoad.startDecode(self.previewPng)

    def DecodePicture(self, PicInfo=None):
        ptr = self.PicLoad.getData()
        if ptr is not None:
            self['pixmap'].instance.setPixmap(ptr)
        return


class sgrabberFilesScreen(Screen):
    skin = '''<screen name="sgrabberFilesScreen" position="center,center" size="1280,720" title="Screenshot Files" flags="wfNoBorder">
                <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ScreenGrabber/images/framesd.png" position="0,0" scale="stretch" size="1280,720" />
                <ePixmap position="40,60" size="600,6" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ScreenGrabber/images/slider.png" alphatest="blend" transparent="1" backgroundColor="#ff000000" />
                <widget name="menu" position="42,72" size="600,500" scrollbarMode="showOnDemand" transparent="1" zPosition="2" font="Regular; 30" itemHeight="50" />
                <widget name="info" position="44,581" zPosition="4" size="1189,55" font="Regular;28" foregroundColor="yellow" transparent="1" halign="center" valign="center" />
                <eLabel backgroundColor="#00ffff00" position="638,680" size="300,6" zPosition="12" />
                <eLabel backgroundColor="#000000ff" position="939,680" size="300,6" zPosition="12" />
                <widget name="pixmap" position="682,126" size="550,350" zPosition="5" alphatest="blend" scale="stretch"/>
                <widget name="ButtonYellowtext" position="638,640" size="300,45" zPosition="11" font="Regular; 30" valign="center" halign="center" backgroundColor="#050c101b" transparent="1" foregroundColor="white" />
                <widget name="ButtonBluetext" position="939,640" size="300,45" zPosition="11" font="Regular; 30" valign="center" halign="center" backgroundColor="#050c101b" transparent="1" foregroundColor="white" />
            </screen>'''

    def __init__(self, session):
        self.skin = sgrabberFilesScreen.skin
        Screen.__init__(self, session)
        list = []
        self['menu'] = MenuList(list)
        self['ButtonBluetext'] = Label(_('Preview'))
        self['ButtonYellowtext'] = Label(_('Delete'))
        self['pixmap'] = Pixmap()
        self['info'] = Label()
        self.PicLoad = ePicLoad()
        self.scale = getScale()
        self.folder = config.plugins.sgrabber.storedir.value + '/screenshots/'
        self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', "DirectionActions"], {'yellow': self.removefile,
                                                                                              'blue': self.key_ok,  # self.onFileAction,
                                                                                              "up": self.up,
                                                                                              "down": self.down,
                                                                                              "left": self.left,
                                                                                              "right": self.right,
                                                                                              'ok': self.key_ok,
                                                                                              'cancel': self.close}, -2)
        self.fillplgfolders()
        # self.onLayoutFinish.append(self.ShowPicture)

    def removefile(self):
        self['info'].setText('')
        try:
            fname = self['menu'].getCurrent()
            filename = self.folder + fname
            if filename is not None:
                remove(filename)
            self.fillplgfolders()
        except:
            self['info'].setText('unable to delete file')

    def onFileAction(self):
        self['info'].setText('')
        try:
            self.session.open(sgrabberPic_Thumb, self.fullpath, 0, self.folder)
        except TypeError as e:
            self['info'].setText('unable to preview file')
            print('error:', e)

    def fillplgfolders(self):
        self['info'].setText('')
        plgfolders = []
        fullpath = []
        if path.exists(self.folder):
            for x in listdir(self.folder):
                if path.isfile(self.folder + x):
                    if x.endswith('.jpg') or x.endswith('.png') or x.endswith('.bmp') or x.endswith('.gif'):
                        plgfolders.append(x)
                        fullpath.append(x)

        self['menu'].setList(plgfolders)
        self.fullpath = fullpath
        '''
        print('files number:', len(plgfolders))
        if len(plgfolders) <= 0:
            msg_text = _("No picture files.\nGrabbing Screenshot first")
            msg_type = MessageBox.TYPE_INFO
            AddNotification(MessageBox, str(msg_text), msg_type, timeout=5)
       '''

    def up(self):
        try:
            self['menu'].up()
            self.ShowPicture()
        except Exception as e:
            print(e)

    def down(self):
        try:
            self['menu'].down()
            self.ShowPicture()
        except Exception as e:
            print(e)

    def left(self):
        try:
            self['menu'].pageUp()
            self.ShowPicture()
        except Exception as e:
            print(e)

    def right(self):
        try:
            self['menu'].pageDown()
            self.ShowPicture()
        except Exception as e:
            print(e)

    def key_ok(self):
        fname = self['menu'].getCurrent()
        filename = self.folder + fname
        print('filename ok=', filename)
        if filename is not None:
            self.session.open(GrabPreview, filename)

    def ShowPicture(self, data=None):
        fname = self['menu'].getCurrent()
        filename = self.folder + fname
        print('key_ok name files:', filename)
        if filename is not None:
            print('data: ', data)
            print('filename: ', filename)
            if self["pixmap"].instance:
                try:
                    width = 550  # size_w
                    height = 350  # size_h
                    self.PicLoad.setPara([width, height, self.scale[0], self.scale[1], 0, 1, "FF000000"])
                    if self.PicLoad.startDecode(filename):
                        self.PicLoad = ePicLoad()
                        try:
                            self.PicLoad.PictureData.get().append(self.DecodePicture)
                        except:
                            self.PicLoad_conn = self.PicLoad.PictureData.connect(self.DecodePicture)
                    return
                except Exception as e:
                    print('set para error:', e)
                return

    def DecodePicture(self, PicInfo=None):
        ptr = self.PicLoad.getData()
        if ptr is not None:
            self["pixmap"].instance.setPixmap(ptr)
            self["pixmap"].instance.show()
        return


class sgrabberScreenGrabberView(Screen):

    def __init__(self, session):
        Screen.__init__(self, session)
        self.session = session

        self.myConsole = Console()
        nowService = session.nav.getCurrentlyPlayingServiceReference()
        self.nowService = nowService
        self.srvName = 'ScreenGrabber'
        self.pictureformat = self.getFilename()
        # init test
        self.filename = self.pictureformat
        print("[sgrabberScreenGrabberView] init grab screenshot to %s" % self.pictureformat)
        cmd = "grab"
        cmdoptiontype = str(config.plugins.sgrabber.items.value)
        cmdoptionsize = str(config.plugins.sgrabber.newsize.value)
        cmdoptionformat = str(config.plugins.sgrabber.formatp.value)
        fixedaspectratio = str(config.plugins.sgrabber.fixedaspectratio.value)  # replace('Disabled', '')
        always43 = str(config.plugins.sgrabber.always43.value)  # .replace('Disabled', '')
        bicubic = str(config.plugins.sgrabber.bicubic.value)  # .replace('Disabled', '')
        if not config.plugins.sgrabber.items.value == "All":
            cmd += ' ' + cmdoptiontype
        if not config.plugins.sgrabber.newsize.value == "default":
            cmd += ' ' + cmdoptionsize
        if not config.plugins.sgrabber.formatp.value == "bmp":
            cmd += ' ' + cmdoptionformat
        if not fixedaspectratio == 'Disabled':
            cmd += ' ' + fixedaspectratio
        if not always43 == 'Disabled':
            cmd += ' ' + always43
        if not bicubic == 'Disabled':
            cmd += ' ' + bicubic
        cmd += " %s" % self.filename
        # grab -r1920 -p -n -l -b /tmp/screenshots/ScreenGrabberPreview.png
        extra_args = (self.filename)
        self.whatPic = self.pictureformat
        self.myConsole.ePopen(cmd, self.gotScreenshot, extra_args)
        # end test

        # w = getDesktop(0).size().width()
        # h = getDesktop(0).size().height()
        # PreviewString = '<screen backgroundColor="#ff000000" flags="wfNoBorder" position="center,center" size="' + str(w) + ',' + str(h) + '" title="Preview" transparent="1">\n'
        # PreviewString += ' <widget name="Picture" position="0,0" size="' + str(w) + ',' + str(h) + '" zPosition="2" alphatest="on" transparent="1"/>\n'
        # PreviewString += ' <eLabel font="Regular;24" backgroundColor="#00640808" halign="center" valign="center" transparent="0" position="17,15" size="300,55" text="Please wait....." zPosition="9" />\n'
        # PreviewString += ' <eLabel font="Regular;24" backgroundColor="#00080808" halign="center" valign="center" transparent="0" position="170,680" size="820,25" text="OK=Save        Exit=Play TV        Green=Setup        Yellow=Files" zPosition="9" />\n'
        # PreviewString += ' <eLabel name="" position="317,15" size="160,55" halign="center" valign="center" transparent="0" cornerRadius="26" font="Regular; 22" zPosition="9" text="OK=Save" backgroundColor="#01080911" foregroundColor="#fcc000" />\n'
        # PreviewString += ' <eLabel name="" position="477,15" size="160,55" halign="center" valign="center" transparent="0" cornerRadius="26" font="Regular; 22" zPosition="9" text="Green=Setup" backgroundColor="#01080911" foregroundColor="#fcc000" />\n'
        # PreviewString += ' <eLabel name="" position="637,15" size="160,55" halign="center" valign="center" transparent="0" cornerRadius="26" font="Regular; 22" zPosition="9" text="Yellow=Files TV" backgroundColor="#01080911" foregroundColor="#fcc000" />\n'
        # PreviewString += ' <eLabel name="" position="797,15" size="160,55" halign="center" valign="center" transparent="0" cornerRadius="26" font="Regular; 22" zPosition="9" text="Exit=Play TV" backgroundColor="#01080911" foregroundColor="#fcc000" />\n'
        # PreviewString += '</screen>\n'
        # prvScreen = PreviewString
        # self.skin = prvScreen

        self.EXscale = getScale()
        self.EXpicload = ePicLoad()
        self['Picture'] = Pixmap()
        try:
            self.EXpicload.PictureData.get().append(self.DecodeAction)
        except:
            self.EXpicload_conn = self.EXpicload.PictureData.connect(self.DecodeAction)

        self['actions'] = ActionMap(['WizardActions', 'ColorActions'], {'ok': self.SavePic,
                                                                        'back': self.dexit,
                                                                        'green': self.showsetup,
                                                                        'yellow': self.showfiles}, -1)
        self.onLayoutFinish.append(self.Show_Picture)

    def getFilename(self):
        now = systime()
        now = datetime.fromtimestamp(now)
        now = now.strftime("%Y-%m-%d_%H-%M-%S")
        fileextension = '.png'
        screenshottime = "screenshot_" + now
        if "-j" in config.plugins.sgrabber.formatp.value:
            fileextension = ".jpg"
        elif config.plugins.sgrabber.formatp.value == "bmp":
            fileextension = ".bmp"
        elif config.plugins.sgrabber.formatp.value == "-p":
            fileextension = ".png"
        picturepath = self.getPicturePath()
        if picturepath.endswith('/'):
            screenshotfile = picturepath + screenshottime + fileextension
        else:
            screenshotfile = picturepath + '/' + screenshottime + fileextension
        return screenshotfile

    def getPicturePath(self):
        picturepath = config.plugins.sgrabber.storedir.value
        if picturepath.endswith('/'):
            picturepath = picturepath + 'screenshots'
        else:
            picturepath = picturepath + '/screenshots'
        try:
            if (path.exists(picturepath) is False):
                makedirs(picturepath)
        except OSError:
            self.session.open(MessageBox, _("Sorry, your device for screenshots is not writeable.\n\nPlease choose another one."), MessageBox.TYPE_INFO, timeout=10)
        return picturepath

    def showsetup(self):
        self.session.open(sgrabberScreenGrabberSetup)
        self.dexit()

    def showfiles(self):
        self.session.open(sgrabberFilesScreen)
        '''
        # self.session.open(GrabPreview, self.filename)
        # self.dexit()
        '''

    def dexit(self):
        self.session.nav.playService(self.nowService)
        self.close()

    def Show_Picture(self):
        if fileExists(self.whatPic):
            self.EXpicload.setPara([self['Picture'].instance.size().width(),
                                    self['Picture'].instance.size().height(),
                                    self.EXscale[0],
                                    self.EXscale[1],
                                    0,
                                    1,
                                    '#00080808'])
            self.EXpicload.startDecode(self.whatPic)

    def DecodeAction(self, pictureInfo=''):
        if fileExists(self.whatPic):
            ptr = self.EXpicload.getData()
            self['Picture'].instance.setPixmap(ptr)

    def gotScreenshot(self, result, retval, extra_args=None):
        print('extra args:', str(extra_args))
        print('retval=', str(retval))
        self.retval = retval
        if extra_args is not None:
            filename = extra_args
            self.session.openWithCallback(self.messagetext, GrabPreview, filename)
            # self.session.open(GrabPreview, filename)

    def messagetext(self, callback=None):
        msg_text = None
        if self.retval == 0:
            msg_text = _("Screenshot successfully saved as:\n%s") % self.filename
            msg_type = MessageBox.TYPE_INFO
        else:
            msg_text = _("Grabbing Screenshot failed !!!")
            msg_type = MessageBox.TYPE_ERROR
        if msg_text:
            AddNotification(MessageBox, str(msg_text), msg_type, timeout=10)
        else:
            pass
        self.dexit()

    def SavePic(self):
        now = systime()
        now = datetime.fromtimestamp(now)
        datestr = now.strftime("%Y-%m-%d_%H-%M-%S")
        # srootfolder = config.plugins.sgrabber.storedir.value
        if path.exists(srootfolder + "/screenshots/"):
            self.folder = srootfolder + "/screenshots/"
        else:
            try:
                if (path.exists(srootfolder + "/screenshots/") is False):
                    makedirs(srootfolder + "/screenshots/")
            except OSError:
                self.session.open(MessageBox, _("Sorry, your device for screenshots is not writeable.\n\nPlease choose another one."), MessageBox.TYPE_INFO, timeout=10)
        try:
            if path.exists(self.folder):
                if self.pictureformat.endswith('jpg'):
                    self.filename = self.folder + self.srvName + '-' + datestr + '.jpg'
                else:
                    self.filename = self.folder + self.srvName + '-' + datestr + '.png'
                copyfile(self.pictureformat, self.filename)
                self.session.open(MessageBox, text=_("Screenshot saved to " + self.filename), type=MessageBox.TYPE_INFO, timeout=3, close_on_any_key=True)
            else:
                self.session.open(MessageBox, text=_('Location not available!'), type=MessageBox.TYPE_ERROR, timeout=5, close_on_any_key=True)
        except:
            self.session.open(MessageBox, text=_('Failed saving file!'), type=MessageBox.TYPE_ERROR, timeout=5, close_on_any_key=True)
        self.dexit()


class classScreenGrabber:

    def __init__(self):
        self.dialog = None
        return

    def gotSession(self, session):
        global globalActionMap
        rcbutton = config.plugins.sgrabber.scut.value
        ScreenGrabber_keymap = '/usr/lib/enigma2/python/Plugins/Extensions/ScreenGrabber/keymaps/' + rcbutton + '_keymap.xml'
        self.session = session
        readKeymap(ScreenGrabber_keymap)
        globalActionMap.actions['ShowScreenGrabber'] = self.ShowHide

    def ShowHide(self):
        self.session.open(sgrabberScreenGrabberView)


class sgrabberScreenGrabberSetup(Screen, ConfigListScreen):
    skin = '''<screen name="sgrabberScreenGrabberSetup" position="center,center" size="1280,720" title="ScreebGrabber setup" flags="wfNoBorder">
                <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ScreenGrabber/images/framesd.png" position="0,0" scale="stretch" size="1280,720" />
                <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ScreenGrabber/images/iconafm.jpg" position="891,132" scale="stretch" size="350,350" zPosition="9" />
                <ePixmap position="40,60" size="820,6" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ScreenGrabber/images/slider.png" alphatest="blend" transparent="1" backgroundColor="#ff000000" />
                <widget name="config" position="42,72" size="820,500" scrollbarMode="showOnDemand" transparent="1" zPosition="2" itemHeight="50" font="Regular;34" />
                <widget name="info" position="41,578" zPosition="4" size="1189,55" font="Regular;30" foregroundColor="yellow" transparent="1" halign="center" valign="center" />
                <eLabel backgroundColor="#00ff0000" position="39,680" size="300,6" zPosition="12" />
                <eLabel backgroundColor="#0000ff00" position="337,680" size="300,6" zPosition="12" />
                <eLabel backgroundColor="#00ffff00" position="638,680" size="300,6" zPosition="12" />
                <eLabel backgroundColor="#000000ff" position="939,680" size="300,6" zPosition="12" />
                <widget source="key_red" render="Label" position="37,640" size="300,45" zPosition="11" font="Regular; 30" valign="center" halign="center" backgroundColor="#050c101b" transparent="1" foregroundColor="white" />
                <widget source="key_green" render="Label" position="338,640" size="300,45" zPosition="11" font="Regular; 30" valign="center" halign="center" backgroundColor="#050c101b" transparent="1" foregroundColor="white" />
                <widget source="key_yellow" render="Label" position="638,640" size="300,45" zPosition="11" font="Regular; 30" valign="center" halign="center" backgroundColor="#050c101b" transparent="1" foregroundColor="white" />
                <widget source="key_blue" render="Label" position="938,640" size="300,45" zPosition="11" font="Regular; 30" valign="center" halign="center" backgroundColor="#050c101b" transparent="1" foregroundColor="white" />
                <eLabel name="" position="892,132" size="350,350" />
            </screen>'''

    def __init__(self, session):
        Screen.__init__(self, session)
        self.session = session
        self.createConfigList()
        ConfigListScreen.__init__(self, self.list, session=self.session, on_change=self.changedEntry)
        self['key_red'] = StaticText(_('Cancel'))
        self['key_green'] = StaticText(_('Save'))
        self['key_yellow'] = StaticText(_('Files'))
        self['key_blue'] = StaticText(_('Files II'))
        self['info'] = Label('ScreenGrabber V.%s mfaraj' % currversion)
        self['setupActions'] = ActionMap(['SetupActions', 'ColorActions'], {'yellow': self.showfiles,
                                                                            'green': self.keySave,
                                                                            'blue': self.keyBlue,
                                                                            'cancel': self.keyClose}, -2)

    def createConfigList(self):
        self.list = []
        self.list.append(getConfigListEntry(_('ScreenShot:'), config.plugins.sgrabber.items))
        self.list.append(getConfigListEntry(_('Storing Folder:'), config.plugins.sgrabber.storedir))
        self.list.append(getConfigListEntry(_('Remote screenshot button:'), config.plugins.sgrabber.scut))
        self.list.append(getConfigListEntry(_('Picture size:'), config.plugins.sgrabber.newsize))
        self.list.append(getConfigListEntry(_('screenshot format/quality:'), config.plugins.sgrabber.formatp))
        self.list.append(getConfigListEntry(_('Fixed Aspect ratio:'), config.plugins.sgrabber.fixedaspectratio))
        self.list.append(getConfigListEntry(_('Fixed Aspect ratio 4:3:'), config.plugins.sgrabber.always43))
        self.list.append(getConfigListEntry(_('Bicubic picture resize:'), config.plugins.sgrabber.bicubic))
        self.onitem = config.plugins.sgrabber.items.value
        self.scut = config.plugins.sgrabber.scut.value

    def keyBlue(self):
        self.session.open(sgrabberFilesScreen)

    def changedEntry(self):
        self.createConfigList()
        self["config"].setList(self.list)

    def showfiles(self):
        if self.checkfolder() is True:
            self.folder = config.plugins.sgrabber.storedir.value + '/screenshots/'
            self.session.open(GrabPreviewII, None, self.folder)
        else:
            self.session.open(MessageBox, text=_('Storing directory is not available'), type=MessageBox.TYPE_INFO, timeout=3, close_on_any_key=True)

    def checkfolder(self):
        strfolder = str(self['config'].list[1][1].value)
        if path.exists(strfolder):
            return True
        else:
            return False

    def keySave(self):
        if self.checkfolder() is True:
            for x in self['config'].list:
                x[1].save()
            configfile.save()
            self.changedEntry()
            if not self.onitem == config.plugins.sgrabber.items.value or not self.scut == config.plugins.sgrabber.scut.value:
                self.session.openWithCallback(self.restartenigma, MessageBox, _('Restart enigma2 to load new settings?'), MessageBox.TYPE_YESNO)
            else:
                self.close(True)
        else:
            self.session.open(MessageBox, text=_('Storing directory is not available'), type=MessageBox.TYPE_INFO, timeout=3, close_on_any_key=True)
            return

    def restartenigma(self, result):
        if result:
            self.session.open(TryQuitMainloop, 3)
        else:
            self.close(True)

    def keyClose(self):
        for x in self['config'].list:
            x[1].cancel()
        self.close(False)


class GrabPreviewII(Screen):
    if (getDesktop(0).size().width()) > 1030:
        skin = """
            <screen flags="wfNoBorder" position="228,57" size="1480,963" title="GrabPreview Explorer" backgroundColor="#00121214">
                <ePixmap position="87,75" size="1280,6" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ScreenGrabber/images/slider.png" alphatest="blend" transparent="1" backgroundColor="#ff000000" />
                <ePixmap position="87,800" size="1280,6" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ScreenGrabber/images/slider.png" alphatest="blend" transparent="1" backgroundColor="#ff000000" />
                <widget name="Picture" position="87,80" size="1280,720" zPosition="1" alphatest="on" />
                <widget name="menu" position="300,807" size="800,102" scrollbarMode="showOnDemand" transparent="1" zPosition="5" font="Regular; 34" itemHeight="50" />
                <widget name="State" font="Regular;32" halign="center" position="85,8" size="1280,70" backgroundColor="#01080911" foregroundColor="#fcc000" transparent="1" zPosition="5" />
                <eLabel name="" position="1350,895" size="55,55" halign="center" valign="center" transparent="0" cornerRadius="26" font="Regular; 22" zPosition="1" text="OK" backgroundColor="#01080911" foregroundColor="#fcc000" />
                <eLabel name="" position="1405,895" size="55,55" halign="center" valign="center" transparent="0" cornerRadius="26" font="Regular; 22" zPosition="1" text="EXIT" backgroundColor="#01080911" foregroundColor="#fcc000" />
                <eLabel backgroundColor="#00ffff00" position="1159,887" size="300,6" zPosition="12" />
                <widget name="ButtonYellowtext" position="1157,848" size="300,45" zPosition="11" font="Regular; 30" valign="center" halign="center" backgroundColor="#050c101b" transparent="1" foregroundColor="white" />
            </screen>"""
    else:
        skin = """
            <screen flags="wfNoBorder" position="320,150" size="1280,720" title="GrabPreview Explorer" backgroundColor="#00121214">
                <ePixmap position="228,70" size="850,6" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ScreenGrabber/images/slider.png" alphatest="blend" transparent="1" backgroundColor="#ff000000" />
                <ePixmap position="230,590" size="850,6" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ScreenGrabber/images/slider.png" alphatest="blend" transparent="1" backgroundColor="#ff000000" />
                <widget name="Picture" position="228,70" size="850,520" zPosition="1" alphatest="on" />
                <widget name="menu" position="355,600" size="600,105" scrollbarMode="showOnDemand" transparent="1" zPosition="5" font="Regular; 24" itemHeight="40" />
                <widget name="State" font="Regular;20" halign="center" position="1,-3" size="1280,70" backgroundColor="#01080911" foregroundColor="#fcc000" transparent="0" zPosition="5" />
                <eLabel name="" position="1025,650" size="55,55" halign="center" valign="center" transparent="0" cornerRadius="26" font="Regular; 22" zPosition="1" text="OK" backgroundColor="#01080911" foregroundColor="#fcc000" />
                <eLabel name="" position="1080,650" size="55,55" halign="center" valign="center" transparent="0" cornerRadius="26" font="Regular; 22" zPosition="1" text="EXIT" backgroundColor="#01080911" foregroundColor="#fcc000" />
                <eLabel backgroundColor="#00ffff00" position="965,643" size="250,6" zPosition="12" />
                <widget name="ButtonYellowtext" position="964,600" size="250,45" zPosition="11" font="Regular; 24" valign="center" halign="center" backgroundColor="#050c101b" transparent="1" foregroundColor="white" />
            </screen>"""

    def __init__(self, session, whatPic=None, whatDir=None):
        self.skin = GrabPreviewII.skin
        Screen.__init__(self, session)
        self.session = session
        self.whatPic = whatPic
        if self.whatPic is None:
            self.whatPic = None
        self.whatDir = whatDir
        print('folder picture is:', self.whatDir)
        print('whatPic picture is:', self.whatPic)
        self.picList = []
        self['menu'] = MenuList(self.picList)
        self.EXscale = getScale()
        self.EXpicload = ePicLoad()
        self["Picture"] = Pixmap()
        self["State"] = Label(_('loading... '))
        self["ButtonYellowtext"] = Label(_('Remove'))
        self["actions"] = ActionMap(["WizardActions", "DirectionActions", 'OkCancelActions', 'ColorActions'],
                                    {"ok": self.key_ok,
                                     "back": self.close,
                                     "cancel": self.close,
                                     'yellow': self.removefile,
                                     "up": self.Pleft,
                                     "down": self.Pright,
                                     "left": self.Pleft,
                                     "right": self.Pright}, -1)
        try:
            self.EXpicload.PictureData.get().append(self.DecodeAction)
        except:
            self.EXpicload_conn = self.EXpicload.PictureData.connect(self.DecodeAction)

        self.onLayoutFinish.append(self.Show_Picture)

    def Show_Picture(self):
        if self.whatDir is not None:
            self["State"].visible = False
            for x in listdir(self.whatDir):
                if path.isfile(self.whatDir + x):
                    # print('Show_Picture what dir+x=', self.whatDir + x)
                    if x.endswith('.jpg') or x.endswith('.png') or x.endswith('.bmp') or x.endswith('.gif'):
                        self.picList.append(x)
                # self.picList = sorted(self.picList)
                self.picList = sorted(self.picList, reverse=True)
            self['menu'].setList(self.picList)
            '''
            # print('number file picList=', len(self.picList))
            # if len(self.picList) <= 0:
                # msg_text = _("No picture files.\nGrabbing Screenshot first")
                # msg_type = MessageBox.TYPE_INFO
                # AddNotification(MessageBox, str(msg_text), msg_type, timeout=5)
            '''
            if len(self.picList) > 0:
                self.viewstr()

    def viewstr(self):
        fname = self['menu'].getCurrent()
        self.whatPic = self.whatDir + fname
        if self["State"].visible is False:
            self["State"].visible = True
            self["State"].setText(_(self.whatPic))
            print('self["State"].visible Show_Picture')
        # print('Show_Picture name first files:', self.whatPic)
        self.EXpicload.setPara([self["Picture"].instance.size().width(), self["Picture"].instance.size().height(), self.EXscale[0], self.EXscale[1], 0, 1, "#002C2C39"])
        self.EXpicload.startDecode(self.whatPic)

    def DecodeAction(self, pictureInfo=""):
        if self.whatPic is not None:
            self["State"].setText(_(self.whatPic))
            ptr = self.EXpicload.getData()
            self["Picture"].instance.setPixmap(ptr)

    def removefile(self):
        try:
            if self.whatDir is not None:
                fname = self['menu'].getCurrent()
                filename = self.whatDir + fname
                if path.isfile(filename):
                    remove(filename)
                newPicList = [x for x in self.picList if x != fname]
                # Imposta la nuova lista aggiornata nel menu
                self.picList = newPicList
                self['menu'].setList(self.picList)
                self['menu'].moveToIndex(0)
                if len(self.picList) > 0:
                    self.viewstr()
        except Exception as e:
            print(e)

    def Pright(self):
        self["menu"].down()
        if len(self.picList) > 0:
            fname = self['menu'].getCurrent()
            self.whatPic = self.whatDir + fname
            print('name pright files:', self.whatPic)
            if self["State"].visible is False:
                # print('self["State"].visible')
                self["State"].visible = True
                self["State"].setText(_('loading... ' + self.whatPic))
        else:
            # print('not self["State"].visible')
            self["State"].setText(_("..."))
            self.session.open(MessageBox, _('No more picture-files.'), MessageBox.TYPE_INFO, timeout=3)

        if self.whatPic is not None:
            self.EXpicload.setPara([self["Picture"].instance.size().width(), self["Picture"].instance.size().height(), self.EXscale[0], self.EXscale[1], 0, 1, "#002C2C39"])
            self.EXpicload.startDecode(self.whatPic)

    def Pleft(self):
        self["menu"].up()
        if len(self.picList) > 0:
            fname = self['menu'].getCurrent()
            self.whatPic = self.whatDir + fname
            # print('name pleft files:', self.whatPic)
            if self["State"].visible is False:
                # print('self["State"].visible')
                self["State"].visible = True
                self["State"].setText(_('loading... ' + self.whatPic))
        else:
            # print('not self["State"].visible')
            self["State"].setText(_("..."))
            self.session.open(MessageBox, _('No more picture-files.'), MessageBox.TYPE_INFO, timeout=3)

        if self.whatPic is not None:
            self.EXpicload.setPara([self["Picture"].instance.size().width(), self["Picture"].instance.size().height(), self.EXscale[0], self.EXscale[1], 0, 1, "#002C2C39"])
            self.EXpicload.startDecode(self.whatPic)

    def key_ok(self):
        if self.whatPic is not None and len(self.picList) > 0:
            fname = self['menu'].getCurrent()
            self.whatPic = self.whatDir + fname
            self.EXpicload.setPara([self["Picture"].instance.size().width(), self["Picture"].instance.size().height(), self.EXscale[0], self.EXscale[1], 0, 1, "#002C2C39"])
            self.EXpicload.startDecode(self.whatPic)


class ConsoleItem:
    def __init__(self, containers, cmd, callback, extra_args, binary=False):
        self.extra_args = extra_args
        self.callback = callback
        self.container = enigma.eConsoleAppContainer()
        self.containers = containers
        self.binary = binary
        # print('self.binary ConsoleItem=', self.binary)
        self.filename = self.getFilename()
        name = cmd
        if name in containers:
            name = str(cmd) + '@' + hex(id(self))
        self.name = name
        containers[name] = self

        if callback is not None:
            self.appResults = []
            self.container.dataAvail.append(self.dataAvailCB)
        self.container.appClosed.append(self.finishedCB)

        if isinstance(cmd, str):
            cmd = [cmd]
        retval = self.container.execute(*cmd)
        if retval:
            self.finishedCB(retval)
        if callback is None:
            try:
                os.waitpid(self.container.getPID(), 0)
            except:
                pass

    def dataAvailCB(self, data):
        self.appResults.append(data)

    def finishedCB(self, retval):
        # print("[Console] finished:", self.name)
        del self.containers[self.name]
        del self.container.dataAvail[:]
        del self.container.appClosed[:]
        self.container = None
        callback = self.callback
        if callback is not None:
            try:
                # Concatenare i risultati in byte
                data = b''.join(self.appResults)
            except Exception as e:
                print(f"[Error] Failed to join appResults: {e}")
                return  # Uscita se l'operazione fallisce
            print(f"[Debug] Data length after join: {len(data)} bytes")
            # Gestione del file binario (immagine)
            try:
                with open(self.filename, "wb") as f:
                    f.write(data)
                print(f"[Debug] Successfully wrote {len(data)} bytes to {self.filename}.")
            except Exception as e:
                print(f"[Error] Failed to write binary data to file: {e}")
            # Passa comunque i dati binari al callback
            callback(data, retval, self.extra_args)
        return

    def getFilename(self):
        now = systime()
        now = datetime.fromtimestamp(now)
        now = now.strftime("%Y-%m-%d_%H-%M-%S")
        fileextension = '.png'
        screenshottime = "screenshot_" + now
        if "-j" in config.plugins.sgrabber.formatp.value:
            fileextension = ".jpg"
        elif config.plugins.sgrabber.formatp.value == "bmp":
            fileextension = ".bmp"
        elif config.plugins.sgrabber.formatp.value == "-p":
            fileextension = ".png"
        picturepath = self.getPicturePath()
        if picturepath.endswith('/'):
            screenshotfile = picturepath + screenshottime + fileextension
        else:
            screenshotfile = picturepath + '/' + screenshottime + fileextension
        return screenshotfile

    def getPicturePath(self):
        picturepath = config.plugins.sgrabber.storedir.value
        if picturepath.endswith('/'):
            picturepath = picturepath + 'screenshots'
        else:
            picturepath = picturepath + '/screenshots'
        try:
            if (path.exists(picturepath) is False):
                makedirs(picturepath)
        except OSError:
            self.session.open(MessageBox, _("Sorry, your device for screenshots is not writeable.\n\nPlease choose another one."), MessageBox.TYPE_INFO, timeout=10)
        return picturepath


class Console:
    """
        Console by default will work with strings on callback.
        If binary data required class shoud be initialized with Console(binary=True)
    """

    def __init__(self, binary=False):
        # Still called appContainers because Network.py accesses it to
        # know if there's still stuff running
        self.appContainers = {}
        self.binary = binary
        print('self.binary console=', self.binary)

    def ePopen(self, cmd, callback=None, extra_args=[]):
        print("[Console] command:", cmd)
        return ConsoleItem(self.appContainers, cmd, callback, extra_args, self.binary)

    def eBatch(self, cmds, callback, extra_args=[], debug=False):
        self.debug = debug
        cmd = cmds.pop(0)
        self.ePopen(cmd, self.eBatchCB, [cmds, callback, extra_args])

    def eBatchCB(self, data, retval, _extra_args):
        (cmds, callback, extra_args) = _extra_args
        if self.debug:
            print('[eBatch] retval=%s, cmds left=%d, data:\n%s' % (retval, len(cmds), data))
        if cmds:
            cmd = cmds.pop(0)
            self.ePopen(cmd, self.eBatchCB, [cmds, callback, extra_args])
        else:
            callback(extra_args)

    def kill(self, name):
        if name in self.appContainers:
            print("[Console] killing: ", name)
            self.appContainers[name].container.kill()

    def killAll(self):
        for name, item in self.appContainers.items():
            print("[Console] killing: ", name)
            item.container.kill()


pScreenGrabber = classScreenGrabber()


def main(session, **kwargs):
    session.open(sgrabberScreenGrabberSetup)


def sessionstart(reason, **kwargs):
    if reason == 0:
        if config.plugins.sgrabber.items.value == 'Disabled':
            pass
        else:
            pScreenGrabber.gotSession(kwargs['session'])


def Plugins(**kwargs):
    list = []
    list.append(PluginDescriptor(where=[PluginDescriptor.WHERE_SESSIONSTART], fnc=sessionstart))
    list.append(PluginDescriptor(icon='screengrabber.png', name='ScreenGrabber', description='Grab screen image V.%s' % currversion, where=PluginDescriptor.WHERE_PLUGINMENU, fnc=main))
    return list
